import logging
from pyramid.security import has_permission
from pyramid.i18n import TranslationStringFactory
from cone.tile import (
    tile,
    Tile,
)
from cone.app.browser.utils import (
    make_url,
    make_query,
)
from cone.ugm.model.groups import Groups
from cone.ugm.browser.listing import PrincipalsListing

logger = logging.getLogger('cone.ugm')
_ = TranslationStringFactory('cone.ugm')


@tile('leftcolumn', 'templates/left_column.pt',
      interface=Groups, permission='view')
class GroupsLeftColumn(Tile):

    add_label = _('add_group', 'Add Group')

    @property
    def add_target(self):
        return make_url(self.request,
                        node=self.model.root['groups'],
                        query=make_query(factory=u'group'))
    
    @property
    def can_add(self):
        return has_permission('add', self.model, self.request)


@tile('rightcolumn', interface=Groups, permission='view')
class GroupsRightColumn(Tile):

    def render(self):
        return u'<div class="column right_column">&nbsp;</div>'


@tile('columnlisting', 'templates/column_listing.pt',
      interface=Groups, permission='view')
class GroupsColumnListing(PrincipalsListing):

    slot = 'leftlisting'
    list_columns = PrincipalsListing.group_list_columns
    listing_attrs = PrincipalsListing.group_attrs
    sort_attr = PrincipalsListing.group_default_sort_column
    css = 'groups'
    batchname = 'leftbatch'
    delete_label = _('delete_group', 'Delete Group')

    @property
    def current_id(self):
        return getattr(self.request, '_curr_listing_id', None)


from pyramid.view import view_config
from cone.ugm.model.users import Users
from cone.ugm.model.utils import ugm_users


@view_config('remote_add_user', accept='application/json',
             renderer='json', context=Users, permission='add')
def remote_add_user(model, request):
    """Add user via remote service.
    
    Returns a JSON response containing success state and a message indicating
    what happened::
    
    {
        success: true, // respective false
        message: 'message'
    }
    
    Expected request parameters:
    
    id
        New user id.
    
    password
        User password to be set initially (optional).
    
    roles
        Comma seperated role names the user initially has.
    
    groups
        Comma seperated groups names the user should initially be member of.
    
    attr.*
        User attributes to be set. I.e. ``attr.mail`` would set the mail
        attribute for newly created user. All request parameters prefixed with
        ``attr`` get checked against user attribute attrmap from settings.
        
        Restrictions - All values, whether single or multi valued, are passed
        as string or list of strings to the create function.
    """
    params = request.params
    id = params.get('id')
    
    if not id:
        return {
            'success': False,
            'message': u"No user ID given.",
        }
    
    users = model.backend
    if id in users:
        return {
            'success': False,
            'message': u"User with given ID already exists.",
        }
    
    password = params.get('password')
    
    add_roles = params.get('roles', '')
    add_roles = [val.strip() for val in add_roles.split(',') if val]
    
    add_groups = params.get('groups', '')
    add_groups = [val.strip() for val in add_groups.split(',') if val]
    
    attrs = dict()
    for key, val in params.items():
        if not key.startswith('attr.'):
            continue
        key = key[key.find('.') + 1:]
        attrs[key] = val
    
    settings = ugm_users(model)
    attrmap = settings.attrs.users_form_attrmap
    exposed = settings.attrs.users_exposed_attributes
    if not exposed:
        exposed = list()
    valid_attrs = attrmap.keys() + exposed
    checked_attrs = dict()
    for key in valid_attrs:
        val = attrs.get(key)
        if not val:
            continue
        checked_attrs[key] = val
    
    try:
        user = users.create(id, **checked_attrs)
        message = u""
        
        from cone.app.security import DEFAULT_ROLES
        available_roles = [role[0] for role in DEFAULT_ROLES]
        for role in add_roles:
            if not role in available_roles:
                message += u"Role '%s' given but inexistent. " % role
                continue
            user.add_role(role)
        
        groups = users.parent.groups
        for group in add_groups:
            if not group in groups:
                message += u"Group '%s' given but inexistent. " % group
                continue
            groups[group].add(id)
        
        users.parent()
        
        if password is not None:
            users.passwd(id, None, password)
        
        message += u"Created user with ID '%s'." % id
        return {
            'success': True,
            'message': message,
        }
    except Exception, e:
        return {
            'success': False,
            'message': str(e),
        }
    finally:
        model.invalidate()


@view_config('remote_delete_user', accept='application/json',
             renderer='json', context=Users, permission='delete')
def remote_delete_user(model, request):
    """Remove user via remote service.
    
    Returns a JSON response containing success state and a message indicating
    what happened::
    
    {
        success: true, // respective false
        message: 'message'
    }
    
    Expected request parameters:
    
    id
        Id of user to delete.
    """
    params = request.params
    id = params.get('id')
    
    if not id:
        return {
            'success': False,
            'message': u"No user ID given.",
        }
    
    users = model.backend
    if not id in users:
        return {
            'success': False,
            'message': u"User with given ID not exists.",
        }
    
    try:
        del users[id]
        users.parent()
        
        message = u"Deleted user with ID '%s'." % id
        return {
            'success': True,
            'message': message,
        }
    except Exception, e:
        return {
            'success': False,
            'message': str(e),
        }
    finally:
        model.invalidate()


from cone.tile import tile, Tile
from cone.app.model import AppRoot
from cone.ugm.browser.columns import Column


@tile('site', interface=AppRoot, permission='view')
class SiteName(Tile):
    def render(self):
        # XXX: make site name configurable
        #settings = ugm_settings(self.model)
        #if 'oshanet' in settings.ldap_gcfg.baseDN:
        #    return "OSHANET"
        #else:
        #    return "CORPORATE"
        return 'SITENAME'


@tile('leftcolumn', interface=AppRoot, permission='view')
class RootLeftColumn(Column):
    
    def render(self):
        return self._render(self.model['users'], 'leftcolumn')


@tile('rightcolumn', interface=AppRoot, permission='view')
class RootRightColumn(Column):
    
    def render(self):
        return self._render(self.model['users'], 'rightcolumn')



from plumber import (
    Part,
    default,
    plumb,
)
from pyramid.security import has_permission
from pyramid.i18n import TranslationStringFactory
from yafowil.base import factory
from cone.ugm.model.utils import ugm_roles

_ = TranslationStringFactory('cone.ugm')


class PrincipalRolesForm(Part):
    
    @default
    @property
    def roles_vocab(self):
        from cone.app.security import DEFAULT_ROLES
        return DEFAULT_ROLES
    
    @default
    @property
    def roles_support(self):
        return ugm_roles(self.model).ldap_roles_container_valid
    
    @plumb
    def prepare(_next, self):
        """Hook after prepare and set 'principal_roles' as selection to 
        ``self.form``.
        """
        _next(self)
        if not self.roles_support:
            return
        if not has_permission('manage', self.model.parent, self.request):
            # XXX: yafowil selection display renderer
            return
        value = []
        if self.action_resource == 'edit':
            value = self.model.model.roles
        roles_widget = factory(
            'field:label:select',
            name='principal_roles',
            value=value,
            props={
                'label': _('roles', 'Roles'),
                'multivalued': True,
                'vocabulary': self.roles_vocab,
                'format': 'single',
                'listing_tag': 'ul',
                'listing_label_position': 'after',
            },
        )
        save_widget = self.form['save']
        self.form.insertbefore(roles_widget, save_widget)
    
    @plumb
    def save(_next, self, widget, data):
        _next(self, widget, data)
        if not self.roles_support:
            return
        if not has_permission('manage', self.model.parent, self.request):
            return
        existing_roles = list()
        if self.action_resource == 'edit':
            principal = self.model.model
            existing_roles = principal.roles
        else:
            id = data.fetch('%s.id' % self.form_name).extracted
            principal = self.model.parent[id].model
        new_roles = data.fetch('%s.principal_roles' % self.form_name).extracted
        removed_roles = list()
        for role in existing_roles:
            if not role in new_roles:
                principal.remove_role(role)
                removed_roles.append(role)
        for role in removed_roles:
            existing_roles.remove(role)
        for role in new_roles:
            if not role in existing_roles:
                principal.add_role(role)
        principal.parent.parent()


import logging
from node.locking import locktree
from node.utils import instance_property
from pyramid.i18n import TranslationStringFactory
from cone.app.model import (
    BaseNode,
    Metadata,
    NodeInfo,
    registerNodeInfo,
    Properties,
)
from cone.ugm.model.user import User
from cone.ugm.model.utils import ugm_backend
from cone.ugm.browser.utils import unquote_slash

logger = logging.getLogger('cone.ugm')
_ = TranslationStringFactory('cone.ugm')


def users_factory():
    return Users()


class Users(BaseNode):
    node_info_name = 'users'

    @instance_property
    def properties(self):
        props = Properties()
        props.in_navtree = True
        return props
    
    @instance_property
    def metadata(self):
        metadata = Metadata()
        metadata.title = _('users_node', 'Users')
        metadata.description = _('users_node_description',
                                 'Container for Users')
        return metadata
    
    @property
    def backend(self):
        return ugm_backend(self).users
    
    @locktree
    def invalidate(self, key=None):
        if key is None:
            del self.backend.parent.storage['users']
            self.clear()
            return
        self.backend.invalidate(key)
        try:
            del self[key]
        except KeyError:
            pass
    
    @locktree
    def __call__(self):
        self.backend()
    
    @locktree
    def __iter__(self):
        try:
            return self.backend.__iter__()
        except Exception, e:
            logger.error(str(e))
            return iter(list())

    iterkeys = __iter__

    @locktree
    def __getitem__(self, name):
        # XXX: temporary hack until paster/webob/pyramid handle urllib
        # quoted slashes in path components
        name = unquote_slash(name)
        try:
            return BaseNode.__getitem__(self, name)
        except KeyError:
            try:
                model = self.backend[name]
            except AttributeError:
                raise KeyError(name)
            user = User(model, name, self)
            self[name] = user
            return user


info = NodeInfo()
info.title = _('users_node', 'Users')
info.description = _('users_node_description',
                     'Container for Users')
info.node = Users
info.addables = ['user']
info.icon = 'cone.ugm.static/images/users16_16.png'
registerNodeInfo('users', info)


import types
import logging
from pyramid.security import has_permission
from pyramid.i18n import TranslationStringFactory
from yafowil.utils import Tag
from cone.tile import Tile
from cone.app.browser.utils import make_url
from cone.ugm.model.utils import (
    ugm_users,
    ugm_groups,
)
from cone.ugm.browser.batch import ColumnBatch

tag = Tag(lambda x: x)

logger = logging.getLogger('cone.ugm')
_ = TranslationStringFactory('cone.ugm')


class ColumnListing(Tile):
    """Abstract column listing.
    """

    current_id = None
    slot = None
    list_columns = []
    css = ''
    # XXX: Batch is effectively disabled, if enabled, the sorting
    # needs to be fixed before slicing
    slicesize = 100000
    batchname = ''

    @property
    def ajax_action(self):
        return 'columnlisting'

    @property
    def sortheader(self):
        ret = list()
        for id, name in self.list_columns:
            ret.append({
                'id': 'sort_%s' % id,
                'default': False,
                'name': name,
            })
        ret[0]['default'] = True
        return ret

    @property
    def batch(self):
        return ColumnBatch(self.batchname,
                           self.query_items,
                           self.slicesize)(self.model, self.request)

    @property
    def slice(self):
        current = int(self.request.params.get('b_page', '0'))
        start = current * self.slicesize
        end = start + self.slicesize
        return start, end

    @property
    def items(self):
        start, end = self.slice
        items = self.query_items
        items = sorted(items, key=lambda x: x['sort_by'].lower())
        return items[start:end]

    @property
    def query_items(self):
        """Return list of dicts like:

        {
            'target': u'http://example.com/foo',
            'content': u'Head Column content',
            'sort_by: u'Sort value',
            'current': False,
            'actions': [
                {
                    'id': 'action_id',
                    'enabled': True,
                    'title': 'Action Title',
                    'target': u'http://example.com/foo',
                }
            ],
        }
        """
        raise NotImplementedError(                          #pragma NO COVERAGE
            u"Abstract ``ColumnListing`` does not "         #pragma NO COVERAGE
            u"implement ``items`` property")                #pragma NO COVERAGE

    def item_content(self, *args):
        ret = u''
        pt = 0
        for arg in args:
            pt += 1
            ret += tag('div', '%s&nbsp;' % arg, class_='sort_col_%i' % pt)
        return ret

    def create_item(self, sort, target, content, current, actions):
        return {
            'sort_by': sort,
            'target': target,
            'content': content,
            'current': current,
            'actions': actions,
        }

    def create_action(self, id, enabled, title, target):
        return {
            'id': id,
            'enabled': enabled,
            'title': title,
            'target': target,
        }

    def extract_raw(self, attrs, name):
        raw = attrs.get(name)
        if type(raw) in [types.ListType, types.TupleType]:
            return raw[0]
        return raw and raw or ''
    
    def calc_list_columns(self, defs):
        ret = list()
        i = 1
        for val in defs.values():
            ret.append(('col_%i' % i, val))
            i += 1
        return ret
    
    @property
    def user_attrs(self):
        """XXX: not generic, move
        """
        settings = ugm_users(self.model)
        return settings.attrs.users_listing_columns.keys()

    @property
    def group_attrs(self):
        """XXX: not generic, move
        """
        settings = ugm_groups(self.model)
        return settings.attrs.groups_listing_columns.keys()

    @property
    def user_list_columns(self):
        """XXX: not generic, move
        """
        settings = ugm_users(self.model)
        defs = settings.attrs.users_listing_columns
        return self.calc_list_columns(defs)

    @property
    def group_list_columns(self):
        """XXX: not generic, move
        """
        settings = ugm_groups(self.model)
        defs = settings.attrs.groups_listing_columns
        return self.calc_list_columns(defs)
    
    @property
    def user_default_sort_column(self):
        """XXX: not generic, move
        """
        settings = ugm_users(self.model)
        attrs = self.user_attrs
        sort = settings.attrs.users_listing_default_column
        if not sort in attrs:
            return attrs[0]
        return sort
    
    @property
    def group_default_sort_column(self):
        """XXX: not generic, move
        """
        settings = ugm_groups(self.model)
        attrs = self.group_attrs
        sort = settings.attrs.groups_listing_default_column
        if not sort in attrs:
            return attrs[0]
        return sort


class PrincipalsListing(ColumnListing):
    """Column listing for principals.
    """
    delete_label = _('delete_principal', 'Delete Principal')
    listing_attrs = []
    sort_attr = None
    
    @property
    def query_items(self):
        can_delete = has_permission('delete', self.model, self.request)
        try:
            attrlist = self.listing_attrs
            sort_attr = self.sort_attr
            ret = list()
            users = self.model.backend
            result = users.search(criteria=None, attrlist=attrlist)
            for key, attrs in result:
                target = make_url(self.request,
                                  node=self.model,
                                  resource=key)
                
                actions = list()
                if can_delete:
                    action_id = 'delete_item'
                    action_title = self.delete_label
                    delete_action = self.create_action(
                        action_id, True, action_title, target)
                    actions = [delete_action]
                
                vals = [self.extract_raw(attrs, attr) for attr in attrlist]
                sort = self.extract_raw(attrs, sort_attr)
                content = self.item_content(*vals)
                current = self.current_id == key
                item = self.create_item(
                    sort, target, content, current, actions)
                ret.append(item)
            return ret
        except Exception, e:
            logger.error(str(e))
        return list()


from StringIO import StringIO
from plumber import (
    Part,
    default,
    plumb,
)
from pyramid.response import Response
from pyramid.security import has_permission
from pyramid.i18n import TranslationStringFactory
from pyramid.view import view_config
from yafowil.base import (
    factory,
    UNSET,
)
from cone.app.browser.utils import make_url
from cone.ugm.model.user import User
from cone.ugm.model.utils import ugm_general

_ = TranslationStringFactory('cone.ugm')


@view_config('portrait_image', context=User, permission='view_portrait')
def portrait_image(model, request):
    """XXX: needs polishing. Return configured default portrait if not set
    on user.
    """
    response = Response()
    cfg = ugm_general(model)
    response.body = model.attrs[cfg.attrs['users_portrait_attr']]
    response.headers['Content-Type'] = 'image/jpeg'
    response.headers['Cache-Control'] = 'max-age=0'
    return response


class PortraitForm(Part):
    """Plumbing part for setting user portrait image.
    """
    
    @default
    @property
    def portrait_support(self):
        cfg = ugm_general(self.model)
        return cfg.attrs['users_portrait'] == 'True'
    
    @plumb
    def prepare(_next, self):
        """Hook after prepare and set 'portrait' as image widget to 
        ``self.form``.
        """
        _next(self)
        if not self.portrait_support:
            return
        model = self.model
        request = self.request
        if has_permission('manage', model.parent, request):
            mode = 'edit'
        else:
            mode = 'display'
        cfg = ugm_general(model)
        image_attr = cfg.attrs['users_portrait_attr']
        image_accept = cfg.attrs['users_portrait_accept']
        image_width = int(cfg.attrs['users_portrait_width'])
        image_height = int(cfg.attrs['users_portrait_height'])
        image_data = model.attrs.get(image_attr)
        if image_data:
            image_value = {
                'file': StringIO(image_data),
                'mimetype': 'image/jpeg',
            }
            image_url = make_url(request, node=model, resource='portrait_image')
        else:
            image_value = UNSET
            resource = 'cone.ugm.static/images/default_portrait.jpg'
            image_url = make_url(request, node=model.root, resource=resource)
        portrait_widget = factory(
            'field:label:error:image',
            name='portrait',
            value=image_value,
            props={
                'label': _('portrait', 'Portrait'),
                'src': image_url,
                'alt': _('portrait', 'Portrait'),
                'accept': image_accept,
                'minsize': (image_width, image_height),
                'crop': {
                    'size': (image_width, image_height),
                    'fitting': True,
                }
            },
            mode=mode)
        save_widget = self.form['save']
        self.form.insertbefore(portrait_widget, save_widget)
    
    @plumb
    def save(_next, self, widget, data):
        if not self.portrait_support or \
          not has_permission('manage', self.model.parent, self.request):
            _next(self, widget, data)
            return
        cfg = ugm_general(self.model)
        image_attr = cfg.attrs['users_portrait_attr']
        portrait = data.fetch('userform.portrait').extracted
        if portrait:
            if portrait['action'] in ['new', 'replace']:
                cropped = portrait['cropped']
                image_data = StringIO()
                cropped.save(image_data, 'jpeg', quality=100)
                image_data.seek(0)
                self.model.attrs[image_attr] = image_data.read()
            if portrait['action'] == 'delete':
                del self.model.attrs[image_attr]
        _next(self, widget, data)


from cone.app.model import Properties
from cone.ugm.model.users import Users
from cone.ugm.model.user import User
from cone.ugm.model.groups import Groups
from cone.ugm.model.group import Group


# user and group factory defaults
factory_defaults = Properties()
factory_defaults.user = dict()
factory_defaults.group = dict()
factory_defaults.role = dict()


from node.locking import locktree
from node.utils import instance_property
from pyramid.i18n import TranslationStringFactory
from cone.app.model import (
    AdapterNode,
    Properties,
    Metadata,
    NodeInfo,
    registerNodeInfo,
)

_ = TranslationStringFactory('cone.ugm')


class User(AdapterNode):
    node_info_name = 'user'
    
    @instance_property
    def properties(self):
        props = Properties()
        return props
    
    @instance_property
    def metadata(self):
        metadata = Metadata()
        metadata.title = _('user_node', 'User')
        metadata.description = _('user_node_description', 'User')
        return metadata
    
    @locktree
    def __call__(self):
        self.model()


info = NodeInfo()
info.title = _('user_node', 'User')
info.description = _('user_node_description', 'User')
info.node = User
info.addables = []
registerNodeInfo('user', info)


from plumber import (
    plumb,
    Part,
)
from cone.app.browser.ajax import ajax_form_fiddle

class AddFormFiddle(Part):
    """Form fiddle plumbing part for user and group add forms.
    """
    
    @plumb
    def __call__(_next, self, model, request):
        ajax_form_fiddle(request, '.right_column', 'inner')
        return _next(self, model, request)


class EditFormFiddle(Part):
    """Form fiddle plumbing part for user and group edit forms.
    """
    
    @plumb
    def __call__(_next, self, model, request):
        ajax_form_fiddle(request, '.principal_form', 'inner')
        return _next(self, model, request)


from pyramid.view import view_config
from pyramid.i18n import (
    TranslationStringFactory,
    get_localizer,
)
from cone.ugm.model.user import User
from cone.ugm.model.group import Group

_ = TranslationStringFactory('cone.ugm')


class Action(object):
    """Abstract action.

    A Subclass must implement ``__call__`` and is supposed to be registetred
    as JSON view, like:

    @view_config(name='action_id', accept='application/json', renderer='json')
    """

    def __init__(self, model, request):
        self.model = model
        self.request = request

    def __call__(self):
        """Perform this action and return JSON response.

        Returns a dict with success flag and return message:
            {
                'success': True,
                'message': 'Return message',
            }
        """
        raise NotImplementedError(u"Abstract action does not implement "
                                  u"``__call__``.")


###############################################################################
# Actions for User application node
###############################################################################

@view_config(name='delete_item', accept='application/json',
             renderer='json', context=User, permission='delete')
class DeleteUserAction(Action):

    def __call__(self):
        """Delete user from database.
        """
        try:
            users = self.model.parent.backend
            uid = self.model.model.name
            del users[uid]
            users()
            self.model.parent.invalidate()
            localizer = get_localizer(self.request)
            message = localizer.translate(
                _('delete_user_from_database',
                  default="Deleted user '${uid}' from database.",
                  mapping={'uid': uid}))
            return {
                'success': True,
                'message': message,
            }
        except Exception, e:
            return {
                'success': False,
                'message': str(e),
            }


@view_config(name='add_item', accept='application/json',
             renderer='json', context=User, permission='manage_membership')
class UserAddToGroupAction(Action):

    def __call__(self):
        """Add user to group.
        """
        params = self.request.params
        group_id = params.get('id')
        if not group_id:
            group_ids = params.getall('id[]')
        else:
            group_ids = [group_id]
        try:
            user = self.model.model
            groups = user.root.groups
            for group_id in group_ids:
                groups[group_id].add(user.name)
            groups()
            self.model.parent.invalidate(user.name)
            localizer = get_localizer(self.request)
            message = localizer.translate(
                _('added_user_to_group',
                  default="Added user '${uid}' to group '${gid}'.",
                  mapping={
                      'uid': user.id,
                      'gid': group_id
                  }))
            return {
                'success': True,
                'message': message,
            }
        except Exception, e:
            return {
                'success': False,
                'message': str(e),
            }


@view_config(name='remove_item', accept='application/json',
             renderer='json', context=User, permission='manage_membership')
class UserRemoveFromGroupAction(Action):

    def __call__(self):
        """Remove user from group.
        """
        params = self.request.params
        group_id = params.get('id')
        if not group_id:
            group_ids = params.getall('id[]')
        else:
            group_ids = [group_id]
        try:
            user = self.model.model
            groups = user.root.groups
            for group_id in group_ids:
                del groups[group_id][user.name]
            groups()
            self.model.parent.invalidate(user.name)
            localizer = get_localizer(self.request)
            message = localizer.translate(
                _('removed_user_from_group',
                  default="Removed user '${uid}' from group '${gid}'.",
                  mapping={
                      'uid': user.id,
                      'gid': group_id
                  }))
            return {
                'success': True,
                'message': message,
            }
        except Exception, e:
            return {
                'success': False,
                'message': str(e),
            }


###############################################################################
# Actions for Group application node
###############################################################################

@view_config(name='delete_item', accept='application/json',
             renderer='json', context=Group, permission='delete')
class DeleteGroupAction(Action):

    def __call__(self):
        """Delete group from database.
        """
        try:
            groups = self.model.parent.backend
            uid = self.model.model.name
            del groups[uid]
            groups()
            self.model.parent.invalidate()
        except Exception, e:
            return {
                'success': False,
                'message': str(e),
            }
        localizer = get_localizer(self.request)
        message = localizer.translate(_('deleted_group',
                                        'Deleted group from database'))
        return {
            'success': True,
            'message': message,
        }


@view_config(name='add_item', accept='application/json',
             renderer='json', context=Group, permission='manage_membership')
class GroupAddUserAction(Action):

    def __call__(self):
        """Add user to group.
        """
        params = self.request.params
        user_id = params.get('id')
        if not user_id:
            user_ids = params.getall('id[]')
        else:
            user_ids = [user_id]
        try:
            group = self.model.model
            for user_id in user_ids:
                group.add(user_id)
            group()
            self.model.parent.invalidate(group.name)
            localizer = get_localizer(self.request)
            message = localizer.translate(_('group_added_user',
                                            'Added user to group'))
            return {
                'success': True,
                'message': message,
            }
        except Exception, e:
            return {
                'success': False,
                'message': str(e),
            }


@view_config(name='remove_item', accept='application/json',
             renderer='json', context=Group, permission='manage_membership')
class GroupRemoveUserAction(Action):

    def __call__(self):
        """Remove user from group.
        """
        params = self.request.params
        user_id = params.get('id')
        if not user_id:
            user_ids = params.getall('id[]')
        else:
            user_ids = [user_id]
        try:
            group = self.model.model
            for user_id in user_ids:
                del group[user_id]
            group()
            self.model.parent.invalidate(group.name)
            localizer = get_localizer(self.request)
            message = localizer.translate(_('group_removed_user',
                                            'Removed user from group'))
            return {
                'success': True,
                'message': message,
            }
        except Exception, e:
            return {
                'success': False,
                'message': str(e),
            }


import os
from node.ext.ldap.ugm import (
    UsersConfig,
    GroupsConfig,
    Ugm,
)
import cone.ugm


def ldap_cfg_file():
    return os.environ['LDAP_CFG_FILE']


def ugm_general(model):
    return model.root['settings']['ugm_general']


def ugm_server(model):
    return model.root['settings']['ugm_server']


def ugm_users(model):
    return model.root['settings']['ugm_users']


def ugm_groups(model):
    return model.root['settings']['ugm_groups']


def ugm_roles(model):
    return model.root['settings']['ugm_roles']


def ugm_backend(model):
    """Always access UGM backend via this function. Currently
    ``cone.app.cfg.auth`` is returned. This might change in future if multiple
    backend management get supported.
    """
    import cone.app
    # return backend if already initialized
    if cone.app.cfg.auth is not None:
        return cone.app.cfg.auth
    import cone.ugm
    return cone.ugm.reset_auth_impl()


import copy
from plumber import plumber
from pyramid.security import has_permission
from pyramid.i18n import TranslationStringFactory
from yafowil.base import (
    ExtractionError,
    UNSET,
)
from cone.tile import (
    tile,
    Tile,
)
from cone.app.browser.utils import (
    make_url,
    make_query,
)
from cone.app.browser.form import Form
from cone.app.browser.authoring import (
    AddPart,
    EditPart,
)
from cone.app.browser.ajax import AjaxAction
from cone.ugm.model.user import User
from cone.ugm.model.utils import (
    ugm_general,
    ugm_users,
)
from cone.ugm.browser import form_field_definitions
from cone.ugm.browser.columns import Column
from cone.ugm.browser.listing import ColumnListing
from cone.ugm.browser.roles import PrincipalRolesForm
from cone.ugm.browser.portrait import PortraitForm
from cone.ugm.browser.authoring import (
    AddFormFiddle,
    EditFormFiddle,
)
from cone.ugm.browser.principal import PrincipalForm
from cone.ugm.browser.expires import ExpirationForm
from cone.ugm.browser.autoincrement import AutoIncrementForm
from webob.exc import HTTPFound

_ = TranslationStringFactory('cone.ugm')


@tile('leftcolumn', interface=User, permission='view')
class UserLeftColumn(Column):

    add_label = _('add_user', 'Add User')
    
    @property
    def can_add(self):
        return has_permission('add', self.model.parent, self.request)

    def render(self):
        setattr(self.request, '_curr_listing_id', self.model.name)
        return self._render(self.model.parent, 'leftcolumn')


@tile('rightcolumn', 'templates/right_column.pt',
      interface=User, permission='view')
class UserRightColumn(Tile):
    
    @property
    def default_widget(self):
        settings = ugm_general(self.model)
        return settings.attrs['default_membership_assignment_widget']


class Groups(object):
    """Descriptor to return principal items for listing

    XXX: check also group.Principals, naming here is sometimes newer
    XXX: speedup!
    """
    def __init__(self,
                 related_only=False,
                 available_only=False):
        self.related_only = related_only
        self.available_only = available_only

    def __get__(self, obj, objtype=None):
        if obj is None:
            return self

        appuser = obj.model
        user = appuser.model
        groups = user.groups
        related_ids = [g.name for g in groups]
        
        # always True if we list members only, otherwise will be set
        # in the loop below
        related = self.related_only
        
        available_only = self.available_only
        
        if related and available_only:
            raise Exception(u"Invalid object settings.")

        if not related:
            # XXX: LDAP query here.
            groups = obj.model.root['groups'].backend.values()
            if available_only:
                groups = [g for g in groups if not g.name in related_ids]
        
        ret = list()
        
        can_change = has_permission(
            'manage_membership', obj.model.parent, obj.request)
        
        attrlist = obj.group_attrs
        sort_attr = obj.group_default_sort_column

        # XXX: These should be the mapped attributes - lack of backend support
        for group in groups:
            id = group.name
            attrs = group.attrs

            # XXX: resource was only set for alluserlisting
            item_target = make_url(obj.request, path=group.path[1:])
            action_query = make_query(id=id)
            action_target = make_url(obj.request,
                                     node=appuser,
                                     query=action_query)

            if not self.related_only:
                related = id in related_ids

            actions = list()
            if can_change:
                action_id = 'add_item'
                action_enabled = not bool(related)
                action_title = _('add_user_to_selected_group',
                                 'Add user to selected group')
                add_item_action = obj.create_action(
                    action_id, action_enabled, action_title, action_target)
                actions.append(add_item_action)

                action_id = 'remove_item'
                action_enabled = bool(related)
                action_title = _('remove_user_from_selected_group',
                                 'Remove user from selected group')
                remove_item_action = obj.create_action(
                    action_id, action_enabled, action_title, action_target)
                actions.append(remove_item_action)
            
            vals = [obj.extract_raw(attrs, attr) for attr in attrlist]
            sort = obj.extract_raw(attrs, sort_attr)
            content = obj.item_content(*vals)
            current = False
            item = obj.create_item(sort, item_target, content, current, actions)
            ret.append(item)
        return ret


@tile('columnlisting', 'templates/column_listing.pt',
      interface=User, permission='view')
class GroupsOfUserColumnListing(ColumnListing):

    slot = 'rightlisting'
    list_columns = ColumnListing.group_list_columns
    css = 'groups'
    query_items = Groups(related_only=True)
    batchname = 'rightbatch'


@tile('allcolumnlisting', 'templates/column_listing.pt',
      interface=User, permission='view')
class AllGroupsColumnListing(ColumnListing):

    slot = 'rightlisting'
    list_columns = ColumnListing.group_list_columns
    css = 'groups'
    query_items = Groups(related_only=False)
    batchname = 'rightbatch'

    @property
    def ajax_action(self):
        return 'allcolumnlisting'


@tile('inoutlisting', 'templates/in_out.pt',
      interface=User, permission='view')
class InOutListing(ColumnListing):
    
    selected_items = Groups(related_only=True)
    available_items = Groups(available_only=True)
    
    @property
    def group_attrs(self):
        settings = ugm_general(self.model)
        return [settings.attrs['group_display_name_attr']]
    
    @property
    def group_default_sort_column(self):
        settings = ugm_general(self.model)
        return settings.attrs['group_display_name_attr']
    
    @property
    def display_control_buttons(self):
        return True
        #settings = ugm_general(self.model)
        #return settings.attrs['controls_membership_assignment_widget']


class UserForm(PrincipalForm):
    
    form_name = 'userform'
    
    @property
    def form_attrmap(self):
        settings = ugm_users(self.model)
        return settings.attrs.users_form_attrmap
    
    @property
    def form_field_definitions(self):
        """Hook optional_login extractor if necessary for form defaults.
        """
        schema = copy.deepcopy(form_field_definitions.user)
        id, login = self._get_auth_attrs()
        if id != login:
            field = schema.get(login, schema['default'])
            if field['chain'].find('*optional_login') == -1:
                field['chain'] = '%s:%s' % (
                    '*optional_login', field['chain'])
                if not field.get('custom'):
                    field['custom'] = dict()
                field['custom']['optional_login'] = \
                    (['context.optional_login'], [], [], [], [])
            schema[login] = field
        return schema

    def exists(self, widget, data):
        id = data.extracted
        if id is UNSET:
            return data.extracted
        if id in self.model.parent.backend:
            message = _('user_already_exists',
                        default="User ${uid} already exists.",
                        mapping={'uid': id})
            raise ExtractionError(message)
        return data.extracted
    
    def optional_login(self, widget, data):
        id, login = self._get_auth_attrs()
        res = self.model.parent.backend.search(criteria={login: data.extracted})
        # no entries found with same login attribute set.
        if not res:
            return data.extracted
        # 1-lenght result, unchange login attribute
        if len(res) == 1:
            if res[0] == self.model.name:
                return data.extracted
        message = _('user_login_not_unique',
                    default="User login ${login} not unique.",
                    mapping={'login': data.extracted})
        raise ExtractionError(message)
    
    def _get_auth_attrs(self):
        config = ugm_users(self.model)
        aliases = config.attrs.users_aliases_attrmap
        return aliases['id'], aliases['login']


@tile('addform', interface=User, permission='add')
class UserAddForm(UserForm, Form):
    __metaclass__ = plumber
    __plumbing__ = (
        AddPart,
        PrincipalRolesForm,
        PortraitForm,
        ExpirationForm,
        AutoIncrementForm,
        AddFormFiddle,
    )
    
    show_heading = False
    show_contextmenu = False

    def save(self, widget, data):
        settings = ugm_users(self.model)
        attrmap = settings.attrs.users_form_attrmap
        extracted = dict()
        for key, val in attrmap.items():
            val = data.fetch('userform.%s' % key).extracted
            if not val:
                continue
            extracted[key] = val
        # possibly extracted by other parts
        # XXX: call next in at the end in all extension parts to reduce
        #      database queries.
        for key, val in self.model.attrs.items():
            extracted[key] = val
        users = self.model.parent.backend
        id = extracted.pop('id')
        password = extracted.pop('userPassword')
        users.create(id, **extracted)
        self.request.environ['next_resource'] = id
        users()
        if password is not UNSET:
            users.passwd(id, None, password)
        self.model.parent.invalidate()
        # XXX: access already added user after invalidation.
        #      if not done, there's some kind of race condition with ajax
        #      continuation. figure out why.
        self.model.parent[id]

    def next(self, request):
        next_resource = self.request.environ.get('next_resource')
        if next_resource:
            url = make_url(request.request,
                           node=self.model,
                           resource=next_resource)
        else:
            url = make_url(request.request, node=self.model)
        if self.ajax_request:
            return [
                AjaxAction(url, 'leftcolumn', 'replace', '.left_column'),
                AjaxAction(url, 'rightcolumn', 'replace', '.right_column'),
            ]
        return HTTPFound(location=url)


@tile('editform', interface=User, permission='edit', strict=False)
class UserEditForm(UserForm, Form):
    __metaclass__ = plumber
    __plumbing__ = (
        EditPart,
        PrincipalRolesForm,
        PortraitForm,
        ExpirationForm,
        EditFormFiddle,
    )
    
    show_heading = False
    show_contextmenu = False

    def save(self, widget, data):
        settings = ugm_users(self.model)
        attrmap = settings.attrs.users_form_attrmap
        for key in attrmap:
            if key in ['id', 'login', 'userPassword']:
                continue
            extracted = data.fetch('userform.%s' % key).extracted            
            if not extracted:
                if key in self.model.attrs:
                    del self.model.attrs[key] 
            else:
                self.model.attrs[key] = extracted
        # set object classes if missing
        ocs = self.model.model.context.attrs['objectClass']
        for oc in settings.attrs.users_object_classes:
            if isinstance(ocs, basestring):
                ocs = [ocs]
            if not oc in ocs:
                ocs.append(oc)
        if ocs != self.model.model.context.attrs['objectClass']:
            self.model.model.context.attrs['objectClass'] = ocs
        password = data.fetch('userform.userPassword').extracted
        self.model.model.context()
        if password is not UNSET:
            id = self.model.name
            self.model.parent.backend.passwd(id, None, password)

    def next(self, request):
        url = make_url(request.request, node=self.model)
        if self.ajax_request:
            return [
                AjaxAction(url, 'leftcolumn', 'replace', '.left_column'),
                AjaxAction(url, 'rightcolumn', 'replace', '.right_column'),
            ]
        return HTTPFound(location=url)